<?php
/*
Description: Samsara content integration plugin for Wordpress 2.0
Date: 2006, February, 19
Author: Curtis Hovey
Author URI: http://curtis.hovey.name/
Version: 0.0.1
*/

/*
	Copyright (C) 2006 Curtis Hovey

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

require_once ('../wp-content/plugins/samsara.php');

// Link to a folder to view & edit the captions
if (isset ($_GET[GALLERY_PARAM])) {
	show_captions ();
}

// Button: save captions
else if (isset ($_POST['update_captions'])) {
	show_captions ();
}

// Button: "Update Option"
else if (isset ($_POST['update_options'])) {
	samsara_update_options ();
}

// Reset Options Button
else if (isset ($_POST['smr_reset_options_button'])) {
	smr_reset_default_options ();

	echo '<div style="background-color: #cfebf7;" id="message" class="updated fade"><p><strong>';
	_e ('Options Reset Successfully', $smr_text_domain);
	echo "</strong></p></div>";

	build_samsara_admin_form ();

}

// Delete Options Button
else if (isset ($_POST['smr_delete_options_button'])) {
	smr_delete_options ();

	echo '<div style="background-color: #cfebf7;" id="message" class="updated fade"><p><strong>';
	_e ('Options Deleted Successfully', $smr_text_domain);
	echo "</strong></p></div>";

	build_samsara_admin_form ();

}

// no action set, lets build the normal form
else {
	build_samsara_admin_form ();
}


// display the samsara admin screen
function build_samsara_admin_form () {
	global $smr_text_domain;	
	
	include ("samsara-admin-view.php");
}


// returns the localized and colorized boolean values
function smr_yes_no ($bool) {
	global $smr_text_domain;
	
	if ($bool) {
		return '<span style="color:#0000cc">' . __('yes', $smr_text_domain) . '</span>';
	}
	else {
		return '<span style="color:#cc0000">' . __('no', $smr_text_domain) . '</span>';
	}
}


// Retrieve information about the currently installed GD library
function smr_gd_status () {
	global $smr_text_domain;
	
	$has_gd = function_exists ("gd_info");
	
	_e ('GD support: ', $smr_text_domain);
	_e (smr_yes_no ($has_gd));
	
	if ($has_gd) {
		_e ('<ul style="margin-left: -10px;">');
		
		$info = gd_info ();
		$keys = array_keys ($info);
		
		for ($i = 0; $i < count ($keys); $i++) {
			if (is_bool ($info[$keys[$i]])) {
				echo '<li style="margin-left: -10px;">' . $keys[$i] .": " . smr_yes_no ($info[$keys[$i]]) . "</li>";
			}
			else {
				echo '<li style="margin-left: -10px;">' . $keys[$i] .": " . $info[$keys[$i]] . "</li>";
			}
		}
		
		_e ("</ul>");
	}
}


function samsara_update_options () {
	global $user_level, $smr_text_domain;

	get_currentuserinfo ();

	if ($user_level < 8) {
		die ( __('Permission denied') );
	}

?>

	<div style="background-color: #cfebf7;" id="message" class="updated fade">
	
<?php

	validate_update_param ('gallery_title', '#^..*+$#');
	validate_update_param ('gallery_folder', '#^[a-zA-Z0-9_][a-zA-Z0-9_\.-]+/$#', '/');
	validate_update_param ('thumb_folder', '#^[a-zA-Z0-9_][a-zA-Z0-9_\.-]+/$#', '/');
	validate_update_param ('slide_folder', '#^[a-zA-Z0-9_][a-zA-Z0-9_\.-]+/$#', '/');
	validate_update_param ('excluded_folders', '#^[a-zA-Z0-9_\., -]*$#');
	validate_update_param ('gallery_param', '#^[a-zA-Z0-9_-]+$#', '/');
	validate_update_param ('sort_alphabetically', '#^(TRUE|FALSE)$#');
	validate_update_param ('thumbs_page', '#^[0-9]{1,3}$#');
	validate_update_param ('slide_width', '#^[0-9]{1,4}$#');
	validate_update_param ('slide_height', '#^[0-9]{1,4}$#');
	validate_update_param ('thumb_width', '#^[0-9]{1,3}$#');
	validate_update_param ('thumb_height', '#^[0-9]{1,3}$#');
	validate_update_param ('use_cache', '#^(TRUE|FALSE)$#');
	validate_update_param ('buffer_size', '#^[0-9]{1,3}M$#', 'M');

?>

		<p>
		<strong><?php _e ('Options Updated Successfully', $smr_text_domain); ?></strong>
		</p>
	</div>
	
<?php

	build_samsara_admin_form ();
}


// validates and updates a parameter using a PERL regular express
function validate_update_param ($param, $pattern, $end = '') {

	if (isset ($_POST[$param])) {
		$value = $_POST[$param];
		
		if ('' != $end && ! preg_match ('#[' . $end . ']$#', $value)) {
			$value = $value . $end;
		}
		
		if (preg_match ($pattern, $value )) {
			update_option ('smr_' . $param, $value);
		}
		else {
			_e ("<p>the value for ${param} was not updated. It must conform to: ${pattern}</p>");
		}
	}
}


// remove samsara from teh database
function smr_delete_options () {
	delete_option ('smr_gallery_title');
	delete_option ('smr_gallery_folder');
	delete_option ('smr_thumb_folder');
	delete_option ('smr_slide_folder');
	delete_option ('smr_gallery_location');
	delete_option ('smr_sort_alphabetically');
	delete_option ('smr_thumbs_page');
	delete_option ('smr_slidewidth');
	delete_option ('smr_slideheight');
	delete_option ('smr_thumbwidth');
	delete_option ('smr_thumbheight');
	delete_option ('smr_enable_cache');
	delete_option ('smr_buffer_size');
	delete_option ('smr_excluded_folders');
}


// reset the database to samara's default options
function smr_reset_default_options () {
	smr_delete_options ();
	samasaraInit ();
}


// displays the pictures and captions of a gallery
function show_captions () {
	include ("samsara-gallery-edit.php");
}

?>