<?php

/*
 * A directory of images representing a gallery of pictures.
 * A gallery may have sub galleries.
 */ 
class Gallery {
	var $_abs_path;
	var $_location;
	var $_image_loc;
	var $_name;
	var $_comment;
	var $_date;
	var $_icon;
	var $_is_visible;
	
	// an associative array of path and location
	var $_thumb;
	
	// associative array of
	// dir_name and Gallery
	var $_galleries;
	
	// associative array of
	// file_name and picture
	var $_pictures;
	
	// an picture that is being viewed
	var $_current_picture;
	
	function Gallery ($location = '/', $abs_path, $name = '') {
		// sanity check, only dirs may be galleries
		if (!$abs_path || !is_dir ($abs_path)) {
			// $this = false;
			return;
		}
		
		// we need to identify dirs with end slashes to ensure that
		// we only crawl permitted dirs and do so only once.
		$this->_abs_path = realpath ($abs_path) . '/';
		$this->_location = str_replace ('http:/', 'http://', str_replace ('//', '/', $location . '/'));
		
		if (isset ($name)) {
			$this->_name = $name;
		}
		
		// set pictures and sub galleries by reading the contents
		// of the gallery dir
		$this->_galleries = array ();
		$this->_pictures = array ();
		
		$this->AssembleGalleriesAndPictures ();
	}
	
	
	/*
	 * accessors and methods
	 */
	 function Name () {
	 	if ('' != $this->_name) {
	 		return $this->_name;
	 	}
	 	
	 	// we can create a crude name based on the dirname,
	 	// but we prefer an assigned name from the gallery's data file
	 	$dirs = split ('/', $this->_abs_path);
	 	$this->_name = str_replace ('_', ' ', $dirs[count ($dirs) - 2]);
	 	
	 	// we don't want to reveal the dirname of the top gallery
	 	if (GALLERY_FOLDER == $this->_name . '/' ) {
	 		$this->_name = GALLERY_TITLE;
	 	}
	 	
	 	return $this->_name;
	 }
	 
	 
	 function Path () {
	 	return $this->_abs_path;
	 }
	 
	 function Location () {
	 	return $this->_location;
	 }
	 
	 
	 function Icon ($random = false) {
	 	if (isset ($_icon)) {
	 		return $_icon;
	 	}
	 	
	 	static $exts;
	 	
	 	if (! isset ($exts )) {
	 		$exts = array ("jpg", "jpeg", "png", "gif");
	 	}
	 	
	 	$dirs = split ('/', $this->_abs_path);
	 	$icon_filebase = $dirs[count ($dirs) - 2];
	 	
	 	foreach ($exts as $ext) {
	 		$icon_path = $this->_abs_path . "icon" . $ext;
	 		
		 	if (is_file ($icon_path) && is_readible ($icon_path)) {
		 		$_icon = $icon_path;
		 		return $icon_file;
		 	}
	 	}
	 	
	 	if ($random) {
	 		return $this->RandomThumbnail ();
	 	}
	 	
	 	return null;
	 }
	 
	 
	 function &Galleries () {
	 	return $this->_galleries;
	 }


	function &Pictures () {
	 	return $this->_pictures;
	 }

	// returns boolean indicating if the Gallery is visible
	// to non admins
	function IsActive () {
		return true;
	}
	
	
	function RandomThumbnail () {
		$picture = GetRandomMapItem ($this->Pictures ());
		
		if ($picture) {
			return $picture->Thumbnail ();
		}
	}
	
	
	/*
	 * worker methods
	 */
	 
	 
	 // fill the subgalleries and pictures arrays
	 function AssembleGalleriesAndPictures () {
		$tmp_gall = array ();
		$tmp_pics = array ();
		
		$dir_content = opendir ($this->_abs_path);

		if (!$dir_content) {
			return;
		}
		
		while (false !== ($file = readdir ($dir_content))) {
			// pages and images are in different trees (virtual and real respectively)
			// so we need to keep diffent paths for the /gallery/ of the page
			// and the wp-gallery of the image.
			$file_path = realpath ($this->_abs_path . $file);
			$file_location = $this->_location . $file;
			$image_location = Currator::photo_uri_to_image_uri ($file_location);

			// check that the file_path is actually inside
			// the directory we are looking in
			$sub_path = str_replace ($this->_abs_path, '', $file_path);
			$has_path_diff = (strlen ($file_path) > strlen ($this->_abs_path)) ? false : true;
			
			if ($sub_path == $file_path ) {
				continue;
			}
			
			// check if the file or dir is a picture or gallery
			if (Picture::is_picture_file ($file_path)) {
				$tmp_pics[$image_location] = $file_path;
			} elseif (Gallery::is_gallery_dir ($file_path)) {	
				$tmp_gall[$file_location . '/'] = $file_path;
			} 
		}
		
		closedir ($dir_content);
		
		// retrieve the caption information from the xml file.
		// we get an associative array of locaton=>caption
		$captions = $this->LoadCaptions ();

		foreach ($tmp_pics as $image_location=>$file_path) {
			$caption = '';
			
			if (array_key_exists ($image_location, $captions)) {
				$caption =& $captions[$image_location];
			}
			
			$this->_pictures[$image_location] = new Picture ($image_location, $file_path, $caption);
		}
		
		foreach ($tmp_gall as $file_location=>$file_path) {
			// prevent recursive processing. This may not be an issue
			if (array_key_exists ($file_location, $this->_galleries)) {
				continue;
			}
			
			$this->_galleries[$file_location] = new Gallery ($file_location, $file_path);
		}
			 
		if (SORT_GALLERIES == 'TRUE') {
			asort ($this->_galleries);
		}
	 }
	 
	 
	 // returns an associative array of image names and captions
	 // the gallery caption is labels gallery
	 function LoadCaptions () {
	 	$captions = array ();
	 	
		$captions_file = $this->_abs_path . CAPTIONS_FILE;
		
		if (! file_exists ($captions_file)) {
			$file = fopen ($captions_file, 'wb');
			
			if (! $file) {
				die ('Cannot create ' . $captions_file);
			}
			
			fwrite ($file, '<?xml version="1.0" encoding="UTF-8"?>' . "\n");
			fwrite ($file, "<data>\n</data>");
			
			fclose ($file);
		}
		
		$data = implode ("", file ($captions_file));
		$caption_parser = new CaptionsXmlToDictionary ($this->_location, true);
		$captions = $caption_parser->parse ($data);
		
		return $captions;
	 }
	 
	 
	 // returns true upon success
	 function UpdateCaptions ($pic_caps) {
	 	if (! is_array ($pic_caps)) {
	 		return false;
	 	}
	 	
	 	$all_set = true;
	 	
	 	foreach ($pic_caps as $loc=>$cap) {
			$success = false;
				 		
	 		$picture =& $this->_pictures[$loc];
	 		
	 		if ($picture) {
	 			$ret_cap = $picture->SetCaption ($cap);
	 			$success = true;
	 		}
	 		
	 		if (! $success) {
	 			$all_set = false;
	 		}
	 	}
	 	
	 	if ($all_set) {
	 		$this->SaveCaptions ();
	 	}
	 	
	 	return $all_set;
	 }
	 
	 
	 function SaveCaptions () {
	 	$photos_xml = "";
	 	
	 	foreach ($this->_pictures as $loc=>$pic) {
	 		$photos_xml .= $pic->Xml ();
	 	}
	 	
	 	$doc_xml = <<<ENDXML
<data>
%s
</data>

ENDXML;
	 	
	 	$xml = sprintf ($doc_xml, $photos_xml);
	 	
	 	$captions_file = $this->_abs_path . CAPTIONS_FILE;
	 	$file = fopen ($captions_file, 'wb');
				
		if (! $file) {
			die ('Cannot create ' . $captions_file);
		}
		
		fwrite ($file, '<?xml version="1.0" encoding="UTF-8"?>' . "\n");
		fwrite ($file, $xml);
		fclose ($file);
	 }
	 
	 /* class methods */
	 
	 // returns true if the context of the view is photos 
	function is_gallery () {
	 	return ("" != Currator::path_query (GALLERY_PARAM));
	}
	
	
	// returns true if the path is to an gallery director
	function is_gallery_dir ($dir_path) {
		static $forbidden_dirs;
		
		if (!is_dir ($dir_path)) {
			return false;
		}
		
		$dir = array_pop (explode ('/', $dir_path));

		// some directorys cannot be galleries because
		// they are system dirs, private dirs, or navigation
		// we may want to move this to the currator
		if (! isset ($forbidden_dirs)) {
			$forbidden_dirs = explode (',', EXCLUDED_FOLDERS);
			
			for ($i = 0; $i < sizeof ($forbidden_dirs); $i++) {
				$forbidden_dirs[$i] = trim ($forbidden_dirs[$i]);
			}
			
			$forbidden_dirs[] = rtrim (SLIDE_FOLDER, '/');
			$forbidden_dirs[] = rtrim (THUMB_FOLDER, '/');
			$forbidden_dirs[] = ".";
			$forbidden_dirs[] = "..";
			$forbidden_dirs[] = "CVS";
			$forbidden_dirs[] = ".svn";
		}
				
		foreach ($forbidden_dirs as $forbidden_dir) {
			if ($dir == $forbidden_dir) {
				return false;
			}
		}
		
		return true;
	}
}


/*
 * returns an associative array of file location and caption
 * when the parse method is called
 */
class CaptionsXmlToDictionary {
	var $photos = array ();
	var $location = '';
	var $is_photo_loc;
	
	var $parser;
	
	var $aPhoto;
	var $aData;
	
	
	function CaptionsXmlToDictionary ($location = '', $is_photo_loc = false) {
		if (isset ($location)) {
			$this->location = $location;
		}
		
		$this->is_photo_loc = $is_photo_loc;
	}
	
	
	function parse ($xml_str) {
		$this->parser = xml_parser_create ();
		
		xml_set_object ($this->parser, $this);
		xml_set_element_handler ($this->parser, "ElOpen", "ElClosed");
		xml_set_character_data_handler ($this->parser, "ElData");
		
		$success = xml_parse ($this->parser, $xml_str);
		
		if (! $success) {
			$msg = xml_error_string (xml_get_error_code ($this->parser));
			$ln = xml_get_current_line_number ($this->parser);
			die (sprintf ("XML error: %s at line %d", $msg, $ln));
		}

		xml_parser_free ($this->parser);

		return $this->photos;
	}
	
	
	function ElOpen ($parser, $name, $attrs) {
		switch ($name) {
			case 'PHOTO' :
				if ($this->is_photo_loc) {
					$this->aPhoto = Currator::photo_uri_to_image_uri ($this->location . $attrs['ID']);
				} else {
					$this->aPhoto = $this->location . $attrs['ID'];
				}
				
				break;
			case 'CAPTION' :
				$this->aData = '';
				break;
		}
	}
	
	
	function ElData ($parser, $data) {
		if (isset ($this->aData)) {
			$this->aData .= trim ($data);
		}
	}
	
	
	function ElClosed ($parser, $name) {
		switch ($name) {
			case 'PHOTO' :
				unset ($this->aPhoto);
				break;
			case 'CAPTION' :
				if (isset ($this->aPhoto)) {
					$this->photos[$this->aPhoto] = $this->aData;
				}
				
				unset ($this->aData);
				break;
		}
	}
}

?>
