<?php

/*
 * An an image file represented as an picture in a Gallery
 */
class Picture {
	var $_abs_path;
	var $_location;
	var $_page;
	var $_pathinfo;
	var $_caption;
	var $_comment;
	var $_date;
	var $_thumb;
	var $_slide;
	var $_is_visible;
	var $_exif;
	
	
	function Picture ($location = '/', $abs_path, $caption = '') {
		// check san val
		if (!$abs_path || !is_file ($abs_path)) {
			// $this = false;
			return;
		}
		
		// set properties	
		$this->_abs_path = $abs_path;
		$this->_location = $location;
		
		if (isset ($caption)) {
			$this->_caption = $caption;
		}
		
		$this->_pathinfo = pathinfo ($this->_abs_path);
		
		$this->_page = Currator::image_uri_to_photo_uri ($location);
	}
	
	
	/*
	 * accessors and methods
	 */	
	
	
	function Location () {
		return $this->_location;
	}
	
	
	function Page () {
		return $this->_page;
	}
	
	 
	function Caption () {
		return $this->_caption;
	}
	
	 
	// gets the thumbnail image location
	function Thumbnail () {
		if ('' != $this->_thumb) {
	 		return $this->_thumb['location'];
	 	}
	 	
	 	// default behavior to lookup/create a cached thumbnail
	 	if (ENABLE_CACHE == "TRUE") {
			$this->_thumb = Picture::LookupCache ($this->_abs_path, $this->_location, true);
			
			return $this->_thumb['location'];
	 	}
	 	
	 	// thumbnail generator URI when caching is not enabled
 		$slide_location = Picture::ImageGenerator (true);
 		
 		$this->_thumb = array ('path'=>'', 'location'=>$thumb_location);
 		
 		return $this->_thumb['location'];
	}


	// gets the slife image location
	function Slide () {
		if ('' != $this->_slide) {
	 		return $this->_slide['location'];
	 	}
	 	
	 	// default behavior to lookup/create a cached thumbnail
	 	if (ENABLE_CACHE == "TRUE") {
			$slide = Picture::LookupCache ($this->_abs_path, $this->_location, false);
			
			if ($slide) {
				$this->_slide = $slide;
			} else {
				return false;
			}
			
			return $this->_slide['location'];
	 	}
	 	
	 	// slide generator URI when caching is not enabled
 		$slide_location = Picture::ImageGenerator (false);
 		
 		$this->_slide = array ('path'=>'', 'location'=>$slide_location);
 		
 		return thumbnail_slide ($this->_slide);
	}
	
	
	function thumbnail_slide (& $image_loc) {
		if ('' != $image_loc) {
	 		return $image_loc['location'];
	 	}
	 	
	 	// default behavior to lookup/create a cached thumbnail
	 	if (ENABLE_CACHE == "TRUE") {
			$image_loc = Picture::LookupCache ($this->_abs_path, $this->_location, false);
			
			return $image_loc['location'];
	 	}
	 	
	 	// slide generator URI when caching is not enabled
 		$i_location = Picture::ImageGenerator (false);
 		
 		$image_loc = array ('path'=>'', 'location'=>$i_location);
 		
 		return $image_loc['location'];
	}
	
	
	// returns the EXIF associative array
	function Exif () {
		if ('' != $this->_exif) {
	 		return $this->_exif;
	 	}	
	
		if ('jpeg' == $this->_pathinfo['extension'] || 'jpg' == $this->_pathinfo['extension']) {
			$verbose = 0;
			$exif_dict = read_exif_data_raw ($this->_abs_path, $verbose);
		}
		
		if (! $exif_dict) {
			return;
		}
		
		// simplify and normalize the exif data
		$major_keys = array_keys ($exif_dict);
		
		foreach ($exif_dict as $major_key=>$major_val) {
			// many values are null, and need not be reported since
			// it is not meaningful
			if (! $major_val) {
				continue;
			}
			
			// some value are arrays that we want to flatten since
			// we only want the meaningful info in the value and key
			if (is_array ($major_val)) {
				foreach ($major_val as $minor_key=>$minor_val) {
					if (! $minor_val | is_array ($minor_val)) {
						continue;
					}
					
					$this->set_exif_key_val ($minor_key, rtrim ($minor_val) );
				}
			} else {
				$this->set_exif_key_val ($major_key, rtrim ($major_val) );
			}
		}
		
		return $this->_exif;
	}
	
	
	function set_exif_key_val ($key, $val) {
		if (! $key | ! $val) {
			return;
		}
		
		$locale_key = Picture::exif_to_locale ($key);
					
		if (! $locale_key) {
			return;
		}
		
		if (substr_count ($key, "FNumber") + substr_count ($key, "Aperture") > 0 ) {
			$val = str_replace ("f ", "f/", $val);
		}
		
		$this->_exif[$locale_key] = $val;
	}
	
	
	function SetCaption ($cap) {
		$this->_caption = htmlentities (str_replace ('\\', '', $cap));
		
		return $this->Caption ();
	}
	
	
	function Xml () {
		$xml = "";
		
		$frag = <<<ENDFRAG
	<photo id="%s">
		<caption>%s</caption>
	</photo>

ENDFRAG;
		
		$basename = basename ($this->_abs_path);
		
		$xml = sprintf ($frag, $basename, $this->Caption ());
	
		return $xml;
	}
	
	/* class methods */
	
	// returns true if the path is to an image file
	function is_picture_file ($file_path) {
		// only readable regular files can be images
		if (! is_file ($file_path) || ! is_readable ($file_path)) {
			return false;
		}
		
		// we assume that files ending in jpg, gif, png , or jpeg
		// are images. A true mime-type check is better though.
		if (! eregi ('^.*\.(jpg|gif|png|jpeg)$', $file_path)) {
			return false;
		}
		
		// images named after the directory are reserved as the default 
		// for the gallery, so we will reject that match
		$parts = explode ('/', $file_path);
		$count = count ($parts);
		
		if (eregi ('^' . $parts[$count - 2] . '\..*$', $parts[$count - 1])) {
			return false;
		}
		
		return true;
	}

	// returns an associate array of the path and location of
	// a cached image
	function LookupCache ($image_path, $image_location, $is_thumb) {
	 	$cache_dir = SLIDE_FOLDER;
	 	
	 	if ($is_thumb) {
	 		$cache_dir = THUMB_FOLDER;
	 	}
	 	
	 	$file_info = pathinfo ($image_path);
		$file_loc = pathinfo ($image_location);

		$cache_info = array ();
	 	
	 	$cache_info['path'] = $file_info['dirname'] . '/' . $cache_dir . $file_info['basename'];
	 	$cache_info['location'] = $file_loc['dirname'] . '/' . $cache_dir . $file_loc['basename'];
	 	
	 	if (! file_exists ($cache_info['path'])) {
			$chwidth = SLIDE_WIDTH;
			$chheight = SLIDE_HEIGHT;
	 		
	 		if ($is_thumb) {
	 			$chwidth = THUMB_WIDTH;
				$chheight = THUMB_HEIGHT;
			}
			
			Picture::CreateCache ($cache_info['path'], $image_path, $chwidth, $chheight);
		}
		
		return $cache_info;
	}
	
	
	// creates a cached version of a scaled image
	function CreateCache ($cache_path, $image_path, $chwidth, $chheight) {
		Picture::ScaleImage ($cache_path, $image_path, $chwidth, $chheight);
	}
	
	
	// send a scaled image to out
	function SendImage ($image_path, $chwidth, $chheight) {
		Picture::ScaleImage ('', $image_path, $chwidth, $chheight);
	}
	
	
	function ScaleImage ($cache_path, $image_path, $chwidth, $chheight) {
		if (! ($image_path & $chwidth & $chheight)) {
			return;
		}

		$is_cache = true;
		
		if ('' == $cache_path) {
			$is_cache = false;
		}
		
		// this will prevent some unshown thumbs
		ini_set ("memory_limit", MEMORY_LIMIT);

		if ($is_cache) {
			// setup the directory for the cached file
			$cache_info = pathinfo ($cache_path);
			
			if (! file_exists ($cache_info['dirname'])) {
				$is_made = mkdir ($cache_info['dirname'], 0775);
				
				if (!$is_made) {
					return false;
				}
			}
			
			if (! is_writable ($cache_info['dirname'])) {
				$is_made = chmod ($cache_info['dirname'], 0775);
				
				if (! $is_made) {
					return false;
				}
			}
		}
		
		// prepare the image
		$img = '';
		$image_info = pathinfo ($image_path);
		
		switch (strtolower ($image_info["extension"])) {
			case "jpeg":
			case "jpg":
				$img = imagecreatefromjpeg ($image_path);
				if (! $is_cache) {
					header ("content-type: image/jpeg");
				}
				break;
			case "gif":
				$img = imagecreatefromgif ($image_path);
				if (! $is_cache) {
					header ("content-type: image/gif");
				}
				break;
			case "png":
				$img = imagecreatefrompng ($image_path);
				if (! $is_cache) {
					header ("content-type: image/png");
				}
				break;
			default:
				return false;
		}
	
		// We have seperate dimensions for thumbnails and slides
		// and we probably must resize the image
		$xratio = $chheight / (imagesx ($img));
		$yratio = $chwidth / (imagesy ($img));
		
		if ($xratio < 1 || $yratio < 1) {
			if ($xratio < $yratio) {
				$resized = imagecreatetruecolor ($chwidth, floor (imagesy ($img) * $xratio));
			}
			else {
				$resized = imagecreatetruecolor (floor (imagesx ($img) * $yratio), $chheight);
			}

			imagecopyresampled ($resized, $img, 0, 0, 0, 0, imagesx ($resized) + 1, imagesy ($resized) + 1, imagesx ($img), imagesy ($img));
			
			$img = $resized;
		}
		
		if ($is_cache) {
			imagejpeg ($img, $cache_path);
		} else {
			imagejpeg ($img);
		}
		
		imagedestroy ($img);		
		
		if ($is_cache && file_exists ($cache_path)) {
			chmod ($cache_path, 0664);
		}
	}

	
	//returns the location of the image generator
	function ImageGenerator ($is_thumb) {
		$operation = 'view_slide';
		
		if ($is_thumb) {
			$operation = 'view_thumbnail';
		}
		
	 	$base_generator = './wp-content/plugins/samsara/samsara-actions.php';
 		$master_gallery_root = get_settings ('siteurl') . GALLERY_ROOT;
 		$gallery_image_loc = str_replace ($master_gallery_root, '', $this->_location);
 		
 		return $base_generator . "?" . GALLERY_PARAM . "=" . $gallery_image_loc . "&amp;operation=${operation}";
	}


	// translate the exif into english
	function exif_to_locale ($key) {
		static $exif_tag;
		
		if (! isset ($exif_tag)) {
			$exif_tag = array (
				// camera
				"ACDComment"=>"ACD Comment"
				,"ImageType"=>"Image Type"
				,"PhotometricInterpret"=>"Photometric Interpret"
				,"ImageDescription"=>"Image Description"
				,"Make"=>"Manufacturer"
				,"Model"=>"Model"
				,"Orientation"=>"Orientation"
				,"SamplePerPixel"=>"Sample per Pixel"
				,"xResolution"=>"x-Resolution"
				,"yResolution"=>"y-Resolution"
				,"PlanarConfig"=>"Planar Config"
				,"ResolutionUnit"=>"Resolution Unit"
				,"DateTime"=>"Date and Time"
				,"Software"=>"Software"
				,"Artist"=>"Artist"
				,"WhitePoint"=>"White Point"
				,"PrimaryChromaticities"=>"Primary Chromaticities"
				,"YCbCrCoefficients"=>"YCbCr Coefficients"
				,"YCbCrPositioning"=>"YCbCr Positioning"
				,"ReferenceBlackWhite"=>"Reference Black and White"
				,"Copyright"=>"Copyright"
				,"PhotoshopSettings"=>"Photoshop Settings"
				,"GPSInfoOffset"=>"GPS Info Offset"
				//,"UserCommentOld"=>"User Comment Old"
				//,"ExifOffset"=>"Exif Offset"
				
				// image
				,"ExposureTime"=>"Exposure Time"
				,"FNumber"=>"F Number"
				,"ExposureProgram"=>"Exposure Program"
				,"SpectralSensitivity"=>"Spectral Sensitivity"
				,"ISOSpeedRatings"=>"ISO Speed Ratings"
				,"ExifVersion"=>"Exif Version"
				,"DateTimeOriginal"=>"Date and Time (Original)"
				,"DateTimedigitized"=>"Date and Time (Digitized)"
				,"ComponentsConfiguration"=>"Components Configuration"
				,"CompressedBitsPerPixel"=>"Compressed Bits per Pixel"
				,"ShutterSpeedValue"=>"Shutter Speed"
				,"ApertureValue"=>"Aperture"
				,"BrightnessValue"=>"Brightness"
				,"ExposureBiasValue"=>"Exposure Bias"
				,"MaxApertureValue"=>"Max Aperture"
				,"SubjectDistance"=>"Subject Distance"
				,"MeteringMode"=>"Metering Mode"
				,"LightSource"=>"Light Source"
				,"Flash"=>"Flash"
				,"FocalLength"=>"Focal Length"
				,"ImageHistory"=>"Image History"
				//,"MakerNote"=>"Maker Note"
				//,"UserComment"=>"User Comment"
				,"SubsecTime"=>"Subsec Time"
				,"SubsecTimeOriginal"=>"Subsec Time Original"
				,"SubsecTimeDigitized"=>"Subsec Time Digitized"
				,"FlashPixVersion"=>"Flash Pix Version"
				,"ColorSpace"=>"Color Space"
				,"ExifImageWidth"=>"Pixel x-Dimension"
				,"ExifImageHeight"=>"Pixel y-Dimension"
				,"RelatedSoundFile"=>"Related Sound File"
				//,"ExifInteroperabilityOffset"=>"Exif Interoperability Offset"
				,"SpacialFreqResponse"=>"Spacial Frequency Response"
				,"FlashEnergy"=>"Flash Energy"
				,"FocalPlaneXResolution"=>"Focal Plane x-Resolution"
				,"FocalPlaneYResolution"=>"Focal Plane y-Resolution"
				,"FocalPlaneResolutionUnit"=>"Focal Plane Resolution Unit"
				,"SubjectLocation"=>"Subject Location"
				,"ExposureIndex"=>"Exposure Index"
				,"SensingMethod"=>"Sensing Method"
				,"FileSource"=>"File Source"
				,"SceneType"=>"Scene Type"
				,"CFAPattern"=>"CFA Pattern"
				,"CustomerRender"=>"Customer Render"
				,"ExposureMode"=>"Exposure Mode"
				,"WhiteBalance"=>"White Balance"
				,"DigitalZoomRatio"=>"Digital Zoom Ratio"
				,"SceneCaptureMode"=>"Scene Capture Mode"
				,"GainControl"=>"Gain Control"
				,"Contrast"=>"Contrast"
				,"Saturation"=>"Saturation"
				,"Sharpness"=>"Sharpness"
				
				// interoperability
				,"InteroperabilityIndex"=>"Interoperability Index"
				,"InteroperabilityVersion"=>"Interoperability Version"
				,"RelatedImageFileFormat"=>"Related Image File Format"
				,"RelatedImageWidth"=>"Related Image Width"
				,"RelatedImageLength"=>"Related Image Length"
				
				//disable embedded thumbnail
				//,"ImageWidth"=>"Image Width"
				//,"ImageLength"=>"Image Length"
				//,"BitsPerSample"=>"Bits per Sample"
				//,"Compression"=>"Compression"
				//,"PhotometricInterpretation"=>"Photometric Interpretation"
				//,"ThumbnailDescription"=>"Thumbnail Description"
				//,"ThumbnailMake"=>"Thumbnail Make"
				//,"ThumbnailModel"=>"Thumbnail Model"
				//,"StripOffsets"=>"Strip Offsets"
				//,"ThumbnailOrientation"=>"Thumbnail Orientation"
				//,"SamplesPerPixel"=>"Samples per Pixel"
				//,"RowsPerStrip"=>"Rows per Strip"
				//,"StripByteCounts"=>"Strip Byte Counts"
				//,"ThumbnailXResolution"=>"Thumbnail x-Resolution"
				//,"ThumbnailYResolution"=>"Thumbnail y-Resolution"
				//,"PlanarConfiguration"=>"Planar Configuration"
				//,"ThumbnailResolutionUnit"=>"Thumbnail Resolution Unit"
				//,"JpegIFOffset"=>"Jpeg IF Offset"
				//,"JpegIFByteCount"=>"Jpeg IF Byte Count"
				//,"YCbCrSubSampling"=>"YCbCr Sub Sampling"
				
				// misc
				//,"SubfileType"=>"Sub file Type"
				//,"TransferFunction"=>"Transfer Function"
				//,"Predictor"=>"Predictor"
				//,"TileWidth"=>"Tile Width"
				//,"TileLength"=>"Tile Length"
				//,"TileOffsets"=>"Tile Offsets"
				//,"TileByteCounts"=>"Tile Byte Counts"
				//,"SubIFDs"=>"Sub IFDs"
				//,"JPEGTables"=>"JPEG Tables"
				//,"CFARepeatPatternDim"=>"CFA Repeat Pattern Dim"
				//,"CFAPattern"=>"CFA Pattern"
				//,"BatteryLevel"=>"Battery Level"
				//,"IPTC/NAA"=>"IPTC/NAA"
				//,"InterColorProfile"=>"Inter Color Profile"
				//,"OECF"=>"OECF"
				//,"Interlace"=>"Interlace"
				//,"TimeZoneOffset"=>"Time Zone Offset"
				//,"SelfTimerMode"=>"Self Timer Mode"
				//,"FlashEnergy"=>"Flash Energy"
				//,"SpatialFrequencyResponse"=>"Spatial Frequency Response"
				//,"Noise"=>"Noise"
				//,"ImageNumber"=>"Image Number"
				//,"SecurityClassification"=>"Security Classification"
				//,"SubjectLocation"=>"Subject Location"
				//,"ExposureIndex"=>"Exposure Index"
				//,"TIFF/EPStandardID"=>"TIFF/EP Standard ID"
				//,"FlashEnergy"=>"Flash Energy"
			);
		}
		
		$locale_key = null;
		
		if (array_key_exists ($key, $exif_tag)) {
			$locale_key = $exif_tag[$key];
		}
		
		return $locale_key;
	}
}

?>
