<?php

/*
 * page template functions
 */



/*
 * returns the URI to the gallery
 */
 function smr_gallery_uri () {
 	return Currator::get_photo_uri ();
 }
 
 
/*
 * tests if the url is to a gallery
 */
 function smr_is_gallery () {
 	return Gallery::is_gallery ();
 }

/*
 * returns the named Gallery of pictures.
 * Optionally takes the absolute path to the dir. Defaults to the current path.
 * Optionally takes a boolean to display the. Defaults to 1 which will show the
 * the requested Gallery and the immediate sub Galleries. 
 */
function smr_single_gallery_title ($prefix = '', $display = true) {
	$gallery = smr_gallery ();
	
	if (!$gallery) {
		return;
	}
	
	$title = $prefix . $gallery->Name ();
	
	if ($display) {
		_e ($title);
	} else {
		return $title;
	}
}


/*
 * returns the current gallery.
 */
function smr_gallery ($location = '') {
	$currator = Currator::GetInstance ();
	
	if (! $currator) {
		return;
	}
	
	if (! $location) {
		$location = Currator::get_photo_uri () . Currator::path_query ('gallery');
	}
	
	$location = str_replace ('//', '/', $location);
	$location = str_replace ('http:/', 'http://', $location);

	$gallery =& $currator->all_galleries[$location];
		
	return $gallery;
}


/*
 * Lists the Galleries of pictures.
 * Optionally takes the absolute path to the dir. Defaults to Gallery root.
 * Optionally takes the depth to displa. Defaults to 1 which will show the
 * the requested Gallery and the immediate sub Galleries. 
 */
function smr_galleries ($location = '', $depth = 1, $icon = false, $display = true) {
	global $edit_captions;
	
	$gallery = smr_gallery ($location);
	
	if (! $gallery) {
		return; 
	}
	
	if ($display && $gallery->Galleries ()) {
		_e ("<ul>\n");
		foreach ($gallery->Galleries () as $location=>$a_gallery) {
			_e ('<li>');
			
			if (1 == $depth) {
				_e ('<span>');
			}
			
			_e ('<a href ="' . $a_gallery->Location () . '">' . $a_gallery->Name () . '</a>');
			
			if ($edit_captions) {
				$path = str_replace (SITE_URL . GALLERY_ROOT, "", $a_gallery->Location ());
				_e (' (<a href ="index.php?page=samsara/samsara-admin.php&amp;' . GALLERY_PARAM . '=' . $path . '">edit &raquo;</a>) ');
			}
		
			if ($icon) {
				_e ('<br /><a href ="' . $a_gallery->Location () . '"><img src="' . $a_gallery->Icon (true) . '" alt="' . $a_gallery->Name () . '" /></a>');
			}
			
			if ($depth > 1) {
				smr_galleries ($location, $depth - 1, $icon, $display);
			}
			
			if (1 == $depth) {
				_e ('</span>');
			}
			
			_e ("</li>\n");
		}
		_e ("</ul>\n");
	} else {
		return $gallery->Galleries ();
	}
}


// returns true if the gallery has subgalleries
function smr_has_galleries ($location = '') {
	$gallery = smr_gallery ($location);
	
	if (! $gallery) {
		return false; 
	}
	
	if (0 == count ($gallery->Galleries ())) {
		return false;
	}
	
	return true;
}

/*
 * returns a random thumb from the selected Gallery or sub Gallery
 * Optionally takes the absolute path to the dir. Defaults to Gallery root.
 */
function smr_random_thumb ($category = '/', $link = false, $display = true) {
	if (null == $category) {
		return;
	}
	
	$pic = smr_random_picture ($category, true);
	
	$markup = '<img src="' . $pic->Thumbnail () . '" alt="" />';
	
	if ($link) {
		$markup = '<a href="' . Currator::image_uri_to_photo_uri ($pic->Location ()) . '">' . $markup . '</a>';
	}
	
	if ($display) {
		_e ($markup);
	} else {
		return $markup;
	}
}


/*
 * returns a random picture from the selected Gallery or sub Gallery
 * Optionally takes the absolute path to the dir. Defaults to Gallery root.
 */
function smr_random_slide ($category = '/', $display = true) {
	$pic = smr_random_picture ($category, false);
	
	if ($display) {
		_e ('<img src="' . $pic->slide () . '" />');
	} else {
		return $thumb;
	}
}


/*
 * returns a random picture from the selected Gallery or sub Gallery
 * Optionally takes the absolute path to the dir. Defaults to Gallery root.
 */
function smr_random_picture ($category = '/', $is_thumb = true) {
	$currator = Currator::GetInstance ();
	
	if (!$currator) {
		return;
	}
	
	$pic = $currator->GetRandomImage ($category);
	
	if (!$pic) {
		return;
	}
	
	return $pic;
}


/*
 * returns true if a there is a gallery that matches the category name.
 */
function smr_has_category_photos ($photo_category = '') {
	if ('' ==  $photo_category) {
		$photo_category = $category_name;
	}
	
	if (! is_dir (GALLERY_ABS_DIR . $photo_category . '/')) {
		return false;
	}
	
	return true;
}


/*
 * returns a gallery of pictures that matches a category name.
 */
function smr_get_category_photos ($photo_category = '') {
	if ('' ==  $photo_category) {
		$photo_category = $category_name;
	}
	
	$location = Currator::get_photo_uri () . $photo_category . '/';
	
	smr_galleries ($location);
}


/*
 * page loops
 */
function smr_page_nav ($prev = '&laquo;', $next = '&raquo;', $with_page_stops = true, $display = true) {
	$page_size = THUMBS_PER_PAGE;
	$page = 1;
	
	if ($page_size != 0) {
		$page = Currator::path_query ('page');
	}
	
	$folder = Currator::path_query ('gallery');

	$the_gallery = smr_gallery ();
	
	if (!$the_gallery) {
		return;
	}
	
	$the_keys = array_keys ($the_gallery->Pictures ());
	$pic_count = count ($the_keys);
	
	// when we view slides, we switch the context of navigation from
	// a set of images (page_size) to 1, We also must resolve the
	// page
	$is_slide = false;
	$pic_loc = Currator::photo_uri_to_image_uri (Currator::create_photo_uri ($folder . Currator::path_query ('image')));
	
	$pic_pos = array_search ($pic_loc, $the_keys, false);
	
	if (! (false === $pic_pos)) {
		$is_slide = true;
	}
	
	if ($is_slide) {
		$page_size = 1;
		$page = $pic_pos + 1;
	}
	
	$page_count = ceil ($pic_count / $page_size);
	
	// next and previous
	$prev_markup = "${prev} prev";
	
	if ($page_size != 0 && $page > 1) {
		if ($is_slide) {
			$uri = Currator::image_uri_to_photo_uri ($the_keys[$pic_pos - 1]);
		} else {
			$uri = Currator::create_photo_uri ($folder, $page - 1);
		}
		$prev_markup = '<a href="' . $uri . '">' . $prev . '</a> <a href="' . $uri . '">prev</a>';
	}
	
	$next_markup = "next ${next}";
	
	if ($page_size != 0 && $page < $page_count) {
		if ($is_slide) {
			$uri = Currator::image_uri_to_photo_uri ($the_keys[$pic_pos + 1]);
		} else {
			$uri = Currator::create_photo_uri ($folder, $page + 1);
		}
		$next_markup = ' <a href="' . $uri . '">next</a> <a href="' . $uri . '">' . $next . '</a>';
	}
	
	// page stops
	$page_stops = "";
	
	if ($with_page_stops  && 1 < $page_count) {
		$pages = range (1, $page_count);
		foreach ($pages as $stop) {
			if ($page == $stop) {
				$page_stops = $page_stops . ' ' . $stop;
			} else {
				if ($is_slide) {
					$uri = Currator::image_uri_to_photo_uri ($the_keys[$stop - 1]);
				} else {
					$uri = Currator::create_photo_uri ($folder, $stop);
				}
				$page_stops = $page_stops . ' <a href="' . $uri . '">' . $stop . '</a>';
			}
		}
	}
	
	if ($display) {
		_e ('<strong>' . $prev_markup . '</strong> ' . $page_stops . ' <strong>' . $next_markup . '</strong>');
	}
}


/*
 * creates a path to the top of the gallery heirarchy.
 */
function smr_navicable_location ($do_echo = true) {
	$nav = split ('/', Currator::path_query ('gallery'));
	array_pop ($nav);

	$dir_depth = sizeof ($nav);
	
	$file = Currator::path_query ('image');
	$path_info = pathinfo ($file);
	
	if (array_key_exists ('extension', $path_info)) {
		$dir_depth++;
	}
	
	$nav_path = '';
	$current = '';	
	
	foreach ($nav as $n) {
		$current .= $n . '/';
		$title = ereg_replace ("_", " ", $n);
		
		if ($title == '') {
			$title = GALLERY_TITLE;
		}
		
		$nav_path = $nav_path . ' &raquo; '; 
		if (1 == $dir_depth) {
			$nav_path = $nav_path . $title;
		}
		else {
			$nav_path = $nav_path . '<a href="' . Currator::create_photo_uri ($current) . '">' . $title . '</a>';
		}
		
		$dir_depth--;
	}

	if (array_key_exists ('extension', $path_info)) {
		$nav_path = $nav_path . ' &raquo; ';
		$nav_path = $nav_path . $path_info['basename'];
	}

	$markup = preg_replace ('/^ &raquo; /', '', $nav_path);
	
	if ($do_echo) {
		echo $markup;
	}

	return $markup;
}
 

function smr_begin_gallery_pages ($location = '') {
	global $the_gallery, $the_keys, $the_picture;
	global $item_idx, $item_end;
	global $page, $rows, $row_idx;
	
	$the_gallery = smr_gallery ($location);
	
	if (! $the_gallery) {
		return;
	}
	
	$the_keys = array_keys ($the_gallery->Pictures ());
	$pic_count = count ($the_keys);

	if (0 == $pic_count) {
		return false;
	}
	
	$pic_idx = -1;
	
	$page_size = THUMBS_PER_PAGE;
	
	if (strstr ($_SERVER['REQUEST_URI'], 'samsara-admin')) {
		$page_size = 0;
	}
	
	$page = 1;
	
	if ($page_size != 0) {
		$page = Currator::path_query ('page');
	}
	
	// to support list presentation we will create an array
	// from the page slice of the total count
	$item_count = ($page_size == 0) ? $pic_count : $page_size;

	if ($item_count > $pic_count) {
		$item_count = $pic_count;
	}
	
	$item_idx = $item_count * ($page - 1) - 1;
	$item_end = ($item_count * $page);
	
	if ($item_end > $item_count) {
		$item_end = $pic_count;
	}
	
	return true;
}


function smr_end_gallery_pages () {
	global $the_gallery, $the_keys, $the_picture, $item_count, $iten_idx, $item_end, $pages, $rows;
	
	$the_gallery = null;
	$the_picture = null;
	$the_keys = null;
	$pages = null;
	$item_idx = null;
	$item_end = null;
	$rows = null;
}


function smr_next_pic () {
	global $the_gallery, $the_picture, $row, $col_idx;
	
	$key = $row[$col_idx];
	
	if (null == $key) {
		return false;
	}
	
	$pictures = $the_gallery->Pictures ();
	$the_picture = $pictures[$key];
	
	return true;
}


function smr_have_pictures () {
	global $item_idx, $item_end;
	
	if ($item_idx < $item_end - 1) {
		return true;
	}
	
	return false;
}

function smr_the_picture ($current = false) {
	global $the_gallery, $the_keys, $the_picture, $item_end, $item_idx;
	
	if ($current) {
		$path = Currator::path_query ('image');
		$file_path = $the_gallery->Path () . $path;
		
		if (! Picture::is_picture_file ($file_path)) {
			$the_picture = null;
			return false;
		} 
		
		$location = Currator::photo_uri_to_image_uri ($the_gallery->Location () . $path);
	} else {
		$item_idx++;
		
		if ($item_idx >= $item_end) {
			$the_picture = null;
			return false;
		}
		
		$location = $the_keys[$item_idx];
	}
	
	$pictures = $the_gallery->Pictures ();
	$the_picture = $pictures[$location];
	
	if (!$the_picture) {
		return false;
	}
	
	return true;
}


function smr_the_page () {
	global $the_picture;
	
	if (! $the_picture) {
		return;
	}
	
	_e ($the_picture->Page ());
}


function smr_the_location ($echo = true) {
	global $the_picture;
	
	if (! $the_picture) {
		return;
	}
	
	if ($echo) {
		_e ($the_picture->Location ());
	}
	else {
		return $the_picture->Location ();
	}
}


function smr_the_thumbnail () {
	global $the_picture;
	
	if (! $the_picture) {
		return;
	}
	
	_e ($the_picture->Thumbnail ());
}


function smr_the_slide () {
	global $the_picture;
	
	if (! $the_picture) {
		return;
	}
	
	_e ($the_picture->Slide ());
}


function smr_the_caption () {
	global $the_picture;
	
	if (! $the_picture) {
		return;
	}
	
	_e ($the_picture->Caption ());
}


function smr_begin_exif () {
	global $the_picture, $exif_keys, $exif_count, $exif_idx;
	
	if (! $the_picture) {
		return;
	}
	
	if (! $the_picture->Exif ()) {
		return false;
	}
	
	$exif_keys = array_keys ($the_picture->Exif ());
	$exif_count = count ($exif_keys);
	$exif_idx = -1;
	
	return true;
}


function smr_end_exif () {
	global $exif_keys, $exif_count, $exif_idx;
	
	$exif_keys = null;
	$exif_count = null;
	$exif_idx = -1;
	$exif_term = null;
	$exif_data = null;
	
	return true;
}


function smr_next_exif () {
	global $the_picture, $exif_keys, $exif_count, $exif_idx, $exif_term, $exif_data;
	
	$exif_idx++;
	
	if ($exif_idx >= $exif_count) {
		return false;
	}
	
	$exif_term = $exif_keys[$exif_idx];
	$exif = $the_picture->Exif ();
	$exif_data = $exif[$exif_term];
	
	return true;
}


function smr_the_exif_term () {
	global $exif_term;
	
	_e ($exif_term);
}


function smr_the_exif_data () {
	global $exif_data;
	
	_e ($exif_data);
}


/*
 * displays the editable picture data in a form
 */
function smr_edit_picture ($label = '(edit this)', $start = '<div>', $finish = '</div>') {
	global $user_level;
	
	get_currentuserinfo ();
	
	if (8 > $user_level) {
		return;
	}
	
	$image = Currator::path_query ('image');
	
	if ('' == $image) {
		return;
	}
	
	$folder = Currator::path_query ('gallery');
	
	$uri = SITE_URL . "/index.php";
	
	include (TEMPLATEPATH . '/edit-gallery-image.php');
}


/*
 * saves the form data about a picture
 */
function smr_save_picture () {

}

/*
 * displays the editable Gallery data in a form
 */
function smr_edit_gallery ($abs_path = '') {

}

/*$gallery
 * saves the form data about a Gallery
 */
function smr_save_gallery () {

}


function get_ui_script () {
	$site_url = get_settings ('siteurl');
	$script_path = get_theme_root_uri () . '/' . get_current_theme () . '/' . 'script.js';
	
	_e ('<script type="text/javascript" src="' . $site_url . '/wp-includes/js/tw-sack.js"></script>' . "\n");
	_e ('<script type="text/javascript" src="' . $script_path . '"></script>');
}
?>